@@ -1,10 +1,13 @@
 import logging
+import os
+import sys
 import warnings
 from datetime import timezone
+from errno import EINVAL, EPIPE
 from inspect import currentframe, getframeinfo
-from io import StringIO
+from io import BytesIO, TextIOWrapper
 from pathlib import Path
-from typing import Iterable, Tuple, Type
+from typing import Iterable, Optional, Tuple, Type
 
 import freezegun
 import pytest
@@ -13,9 +16,19 @@ from streamlink import logger
 from streamlink.exceptions import StreamlinkDeprecationWarning, StreamlinkWarning
 
 
+def getvalue(output: TextIOWrapper, size: int = -1):
+    output.seek(0)
+
+    return output.read(size)
+
+
 @pytest.fixture()
-def output():
-    return StringIO()
+def output(request: pytest.FixtureRequest):
+    params = getattr(request, "param", {})
+    params.setdefault("encoding", "utf-8")
+    output = TextIOWrapper(BytesIO(), **params)
+
+    return output
 
 
 @pytest.fixture()
@@ -24,7 +37,7 @@ def logfile(tmp_path: Path) -> str:
 
 
 @pytest.fixture()
-def log(request: pytest.FixtureRequest, monkeypatch: pytest.MonkeyPatch, output: StringIO):
+def log(request: pytest.FixtureRequest, monkeypatch: pytest.MonkeyPatch, output: TextIOWrapper):
     params = getattr(request, "param", {})
     params.setdefault("format", "[{name}][{levelname}] {message}")
     params.setdefault("style", "{")
@@ -32,13 +45,19 @@ def log(request: pytest.FixtureRequest, monkeypatch: pytest.MonkeyPatch, output:
     if "logfile" in request.fixturenames:
         params["filename"] = request.getfixturevalue("logfile")
 
+    stream: Optional[TextIOWrapper] = output
+    if not params.pop("stdout", True):
+        stream = None
+    if not params.pop("stderr", True):
+        monkeypatch.setattr("sys.stderr", None)
+
     fakeroot = logging.getLogger("streamlink.test")
 
     monkeypatch.setattr("streamlink.logger.root", fakeroot)
     monkeypatch.setattr("streamlink.utils.times.LOCAL", timezone.utc)
 
-    handler = logger.basicConfig(stream=output, **params)
-    assert isinstance(handler, logging.Handler)
+    handler = logger.basicConfig(stream=stream, **params)
+    assert isinstance(handler, logging.StreamHandler)
 
     yield fakeroot
 
@@ -56,7 +75,7 @@ class TestLogging:
         request: pytest.FixtureRequest,
         monkeypatch: pytest.MonkeyPatch,
         log: logging.Logger,
-        output: StringIO,
+        output: TextIOWrapper,
     ):
         params = getattr(request, "param", {})
 
@@ -89,7 +108,7 @@ class TestLogging:
     def test_default_level(self):
         assert logging.getLogger("streamlink").level == logger.WARNING
 
-    def test_level(self, log: logging.Logger, output: StringIO):
+    def test_level(self, log: logging.Logger, output: TextIOWrapper):
         log.setLevel("info")
         log.debug("test")
         output.seek(0)
@@ -99,7 +118,7 @@ class TestLogging:
         log.debug("test")
         assert output.tell() != 0
 
-    def test_level_none(self, log: logging.Logger, output: StringIO):
+    def test_level_none(self, log: logging.Logger, output: TextIOWrapper):
         log.setLevel("none")
         log.critical("test")
         log.error("test")
@@ -108,12 +127,12 @@ class TestLogging:
         log.debug("test")
         log.trace("test")  # type: ignore[attr-defined]
         log.trace("paranoid")  # type: ignore[attr-defined]
-        assert not output.getvalue()
+        assert not getvalue(output)
 
-    def test_output(self, log: logging.Logger, output: StringIO):
+    def test_output(self, log: logging.Logger, output: TextIOWrapper):
         log.setLevel("debug")
         log.debug("test")
-        assert output.getvalue() == "[test][debug] test\n"
+        assert getvalue(output) == "[test][debug] test\n"
 
     @pytest.mark.parametrize(("loglevel", "calllevel", "expected"), [
         (logger.DEBUG, logger.TRACE, ""),
@@ -123,10 +142,10 @@ class TestLogging:
         (logger.ALL, logger.ALL, "[test][all] test\n"),
         (logger.ALL, logger.TRACE, "[test][trace] test\n"),
     ])
-    def test_custom_output(self, log: logging.Logger, output: StringIO, loglevel: int, calllevel: int, expected: str):
+    def test_custom_output(self, log: logging.Logger, output: TextIOWrapper, loglevel: int, calllevel: int, expected: str):
         log.setLevel(loglevel)
         log.log(calllevel, "test")
-        assert output.getvalue() == expected
+        assert getvalue(output) == expected
 
     # https://github.com/streamlink/streamlink/issues/4862
     @pytest.mark.parametrize(("level", "levelname"), [
@@ -147,52 +166,148 @@ class TestLogging:
         (logger.DEBUG, ""),
         (logger.INFO, "[test][info] foo\n[test][info] bar\n"),
     ])
-    def test_iter(self, log: logger.StreamlinkLogger, output: StringIO, level: int, expected: str):
+    def test_iter(self, log: logger.StreamlinkLogger, output: TextIOWrapper, level: int, expected: str):
         def iterator():
             yield "foo"
             yield "bar"
 
         log.setLevel(logger.INFO)
         assert list(log.iter(level, iterator())) == ["foo", "bar"]
-        assert output.getvalue() == expected
+        assert getvalue(output) == expected
 
     @pytest.mark.parametrize("log", [
         {"style": "%", "format": "[%(name)s][%(levelname)s] %(message)s"},
     ], indirect=True)
-    def test_style_percent(self, log: logging.Logger, output: StringIO):
+    def test_style_percent(self, log: logging.Logger, output: TextIOWrapper):
         log.setLevel("info")
         log.info("test")
-        assert output.getvalue() == "[test][info] test\n"
+        assert getvalue(output) == "[test][info] test\n"
 
     @pytest.mark.parametrize("log_failure", [{"style": "invalid"}], indirect=True)
     def test_style_invalid(self, log_failure):
         assert type(log_failure) is ValueError
-        assert str(log_failure) == "Only {} and % formatting styles are supported"
+        assert str(log_failure) == "Style must be one of: %,{,$"
 
     @freezegun.freeze_time("2000-01-02T03:04:05.123456Z")
     @pytest.mark.parametrize("log", [
         {"format": "[{asctime}][{name}][{levelname}] {message}"},
     ], indirect=True)
-    def test_datefmt_default(self, log: logging.Logger, output: StringIO):
+    def test_datefmt_default(self, log: logging.Logger, output: TextIOWrapper):
         log.setLevel("info")
         log.info("test")
-        assert output.getvalue() == "[03:04:05][test][info] test\n"
+        assert getvalue(output) == "[03:04:05][test][info] test\n"
 
     @freezegun.freeze_time("2000-01-02T03:04:05.123456Z")
     @pytest.mark.parametrize("log", [
         {"format": "[{asctime}][{name}][{levelname}] {message}", "datefmt": "%H:%M:%S.%f%z"},
     ], indirect=True)
-    def test_datefmt_custom(self, log: logging.Logger, output: StringIO):
+    def test_datefmt_custom(self, log: logging.Logger, output: TextIOWrapper):
         log.setLevel("info")
         log.info("test")
-        assert output.getvalue() == "[03:04:05.123456+0000][test][info] test\n"
+        assert getvalue(output) == "[03:04:05.123456+0000][test][info] test\n"
 
-    def test_logfile(self, logfile: str, log: logging.Logger, output: StringIO):
+    @pytest.mark.parametrize(("output", "expected"), [
+        pytest.param(
+            {"encoding": "utf-8"},
+            "Bär: 🐻",
+            id="utf-8",
+        ),
+        pytest.param(
+            {"encoding": "ascii"},
+            "B\\xe4r: \\U0001f43b",
+            id="ascii",
+        ),
+        pytest.param(
+            {"encoding": "iso-8859-1"},
+            "Bär: \\U0001f43b",
+            id="iso-8859-1",
+        ),
+        pytest.param(
+            {"encoding": "shift_jis"},
+            "B\\xe4r: \\U0001f43b",
+            id="shift_jis",
+        ),
+    ], indirect=["output"])
+    def test_logstream_encoding(self, log: logging.Logger, output: TextIOWrapper, expected: str):
         log.setLevel("info")
-        log.info("Hello world, Γειά σου Κόσμε, こんにちは世界")
+        log.info("Bär: 🐻")
+        assert getvalue(output) == f"[test][info] {expected}\n"
+
+    def test_logstream_switch(self, log: logging.Logger, output: TextIOWrapper):
+        output_ascii = TextIOWrapper(BytesIO(), encoding="ascii", errors="strict")
+        log.setLevel("info")
+
+        log.info("Bär: 🐻")
+        assert getvalue(output) == "[test][info] Bär: 🐻\n"
+        assert getvalue(output_ascii) == ""
+
+        assert isinstance(log.handlers[0], logging.StreamHandler)
+        # noinspection PyUnresolvedReferences
+        log.handlers[0].setStream(output_ascii)
+
+        log.info("Bär: 🐻")
+        assert getvalue(output) == "[test][info] Bär: 🐻\n"
+        assert getvalue(output_ascii) == "[test][info] B\\xe4r: \\U0001f43b\n"
+
+    @pytest.mark.parametrize(
+        "log",
+        [pytest.param({"stdout": False}, id="no-stdout")],
+        indirect=["log"],
+    )
+    def test_no_stdout(self, log: logging.Logger):
+        assert log.handlers
+        handler = log.handlers[0]
+        assert isinstance(handler, logging.StreamHandler)
+        assert handler.stream is sys.stderr
+
+    @pytest.mark.parametrize(
+        "log",
+        [pytest.param({"stdout": False, "stderr": False}, id="no-stdout-no-stderr")],
+        indirect=["log"],
+    )
+    def test_no_stdout_no_stderr(self, log: logging.Logger):
+        assert log.handlers
+        handler = log.handlers[0]
+        assert isinstance(handler, logging.FileHandler)
+        assert handler.stream.name.endswith(os.devnull)
+
+    @pytest.mark.parametrize(
+        "errno",
+        [
+            pytest.param(EPIPE, id="EPIPE", marks=pytest.mark.posix_only),
+            pytest.param(EINVAL, id="EINVAL", marks=pytest.mark.windows_only),
+        ],
+    )
+    def test_brokenpipeerror(
+        self,
+        monkeypatch: pytest.MonkeyPatch,
+        capsys: pytest.CaptureFixture[str],
+        log: logging.Logger,
+        errno: int,
+    ):
+        def flush(*_, **__):
+            exception = OSError()
+            exception.errno = errno
+            raise exception
+
+        streamhandler = log.handlers[0]
+        assert isinstance(streamhandler, logging.StreamHandler)
+        monkeypatch.setattr(streamhandler.stream, "flush", flush)
+
+        log.setLevel("info")
+        log.info("foo")
+
+        # logging.StreamHandler will write emit()/flush() errors to stderr via handleError()
+        out, err = capsys.readouterr()
+        assert not out
+        assert not err
+
+    def test_logfile(self, logfile: str, log: logging.Logger, output: TextIOWrapper):
+        log.setLevel("info")
+        log.info("Hello world, Γειά σου Κόσμε, こんにちは世界")  # noqa: RUF001
         log.handlers[0].flush()
         with open(logfile, "r", encoding="utf-8") as fh:
-            assert fh.read() == "[test][info] Hello world, Γειά σου Κόσμε, こんにちは世界\n"
+            assert fh.read() == "[test][info] Hello world, Γειά σου Κόσμε, こんにちは世界\n"  # noqa: RUF001
 
 
 class TestCaptureWarnings:
@@ -210,10 +325,10 @@ class TestCaptureWarnings:
 
         return lineno
 
-    def test_no_capture(self, log: logging.Logger, output: StringIO):
+    def test_no_capture(self, log: logging.Logger, output: TextIOWrapper):
         with pytest.warns(UserWarning, match=r"^Test warning$"):
             self._warn([("Test warning", UserWarning)])
-        assert output.getvalue() == ""
+        assert getvalue(output) == ""
 
     @pytest.mark.parametrize("log", [{"capture_warnings": True}], indirect=["log"])
     @pytest.mark.parametrize(("warning", "expected", "origin"), [
@@ -227,7 +342,7 @@ class TestCaptureWarnings:
         self,
         recwarn: pytest.WarningsRecorder,
         log: logging.Logger,
-        output: StringIO,
+        output: TextIOWrapper,
         warning: Tuple[str, Type[Warning]],
         expected: str,
         origin: bool,
@@ -235,7 +350,7 @@ class TestCaptureWarnings:
         lineno = self._warn([warning])
         expected += f"  {__file__}:{lineno}\n" if origin else ""
         assert recwarn.list == []
-        assert output.getvalue() == expected
+        assert getvalue(output) == expected
 
     @pytest.mark.parametrize("log", [{"capture_warnings": True}], indirect=["log"])
     def test_capture_logrecord(
@@ -256,11 +371,11 @@ class TestCaptureWarnings:
         self,
         recwarn: pytest.WarningsRecorder,
         log: logging.Logger,
-        output: StringIO,
+        output: TextIOWrapper,
     ):
         lineno = self._warn([("foo", DeprecationWarning), ("bar", FutureWarning)])
         assert recwarn.list == []
-        assert output.getvalue() == (
+        assert getvalue(output) == (
             f"[warnings][deprecationwarning] foo\n  {__file__}:{lineno}\n"
             + f"[warnings][futurewarning] bar\n  {__file__}:{lineno}\n"
         )
@@ -270,11 +385,11 @@ class TestCaptureWarnings:
         self,
         recwarn: pytest.WarningsRecorder,
         log: logging.Logger,
-        output: StringIO,
+        output: TextIOWrapper,
     ):
         lineno = self._warn([("foo", UserWarning), ("foo", UserWarning)], "once")
         assert recwarn.list == []
-        assert output.getvalue() == f"[warnings][userwarning] foo\n  {__file__}:{lineno}\n"
+        assert getvalue(output) == f"[warnings][userwarning] foo\n  {__file__}:{lineno}\n"
 
     @pytest.mark.parametrize("log", [{"capture_warnings": True}], indirect=["log"])
     @pytest.mark.parametrize("warning", [
@@ -286,9 +401,9 @@ class TestCaptureWarnings:
         self,
         recwarn: pytest.WarningsRecorder,
         log: logging.Logger,
-        output: StringIO,
+        output: TextIOWrapper,
         warning: Tuple[str, Type[Warning]],
     ):
         self._warn([warning], filterwarnings="ignore")
         assert recwarn.list == []
-        assert output.getvalue() == ""
+        assert getvalue(output) == ""
